/*
 * DisplayPanel.java
 *
 * The logic for the animation timer for the algorithm visualization program.
 *
 * (c) 2013 Vijay Ramachandran
 *  2014 Modified by Elodie Fourquet
 *
 * COSC 102, Colgate University
 *
 * IMPORTANT:
 *
 *      YOU MUST MODIFY THIS FILE TO COMPLETE YOUR ASSIGNMENT.
 *
 *      On lines 50 and 62, there are branches for creating the visualization of Mergesort and Quicksort.
 *      Once you have completed these, change those branches as instructed.
 *
 */


import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

public class DisplayPanel extends JPanel implements ActionListener {
  
  private static final Color BLUE =  new Color(100, 149, 237);
  private int delay = 20;
  private double delayScale = 0.25;
  private Timer timer;
  
  private AlgViz av;
  
  // constructor for the panel
  public DisplayPanel() {
    super();
    setOpaque(true);
    setBackground(BLUE);
    
    
    timer = new Timer(delay, this);
  }
  
  // called after validate so size is determined 
  public void createViz(String type) {
    
    if (type.equals("_welcome")) {
      av = new TextViz("Choose an algorithm and then click \"Resume\" to start.");
    }
    
    else if (type.equals("_walk")) {
      av = new WalkViz(this.getWidth(), this.getHeight());
    }
    
    else if (type.equals("_mergesort")) {
      /* replace this once you are done with mergesort
       * 
       for example:
       av = new MergeSortViz(width, height);
       
       you can use the name of any class that extends AlgViz
       
       */
      av = new TextViz("Not yet implemented.");
    }
    
    else if (type.equals("_quicksort")) {
      /* replace this once you are done with quicksort
       * 
       for example:
       av = new QuickSortViz(width, height);
       
       you can use the name of any class that extends AlgViz
       
       */
      av = new TextViz("Not yet implemented.");
    }
    
    else {
      av = new TextViz("Error creating display panel.");
    }
    
    repaint();
    SwingUtilities.invokeLater(av);
  }
  
  // this function is called when the timer fires
  public void actionPerformed(ActionEvent e) {
    av.vizTimer(this.getWidth(), this.getHeight());
    repaint();
  }
  
  public void paintComponent(Graphics g) {
    
    super.paintComponent(g);
    av.vizPaint(g, this.getWidth(), this.getHeight());
  }
  
  public void terminate() {
    timer.stop();
  }
  
  public void menuAction(String cmd) {
    if (cmd.equals("!resume"))
      timer.start();
    
    else if (cmd.equals("!pause"))
      timer.stop();
    
    else if (cmd.equals("!slower")) {
      delay = (int) ((double) delay * (1.0 + delayScale));
      timer.setDelay(delay);
    }
    
    else if (cmd.equals("!faster")) {
      delay = (int) ((double) delay * (1.0 - delayScale));
      if (delay < 2)
        delay = 2;
      timer.setDelay(delay);
    }
  }
}
