/*
 * SortViz.java
 *
 * Main file for the algorithm visualization program.
 *
 * (c) 2013 Vijay Ramachandran
 *  2014 Modified by Elodie Fourquet
 * COSC 102, Colgate University
 *
 * No need to modify this file.
 */

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

public class SortViz extends JFrame implements ActionListener {
  private static final int INITIAL_SIZE = 400;
  
  // constructor for the window, which sets up the menu bar and panel
  public SortViz() {
    super("Algorithm Visualization");
    setDefaultCloseOperation(EXIT_ON_CLOSE);
    
    // begin menu bar setup ...
    
    JMenuBar menuBar = new JMenuBar();
    
    JMenu menu;
    JMenuItem item;
    
    menu = new JMenu("Algorithm");
    menuBar.add(menu);
    
    item = new JMenuItem("Random Walk");
    item.setActionCommand("_walk");
    item.addActionListener(this);
    menu.add(item);
    
    menu.add(new JSeparator());
    
    item = new JMenuItem("Mergesort");
    item.setActionCommand("_mergesort");
    item.addActionListener(this);
    menu.add(item);
    
    item = new JMenuItem("Quicksort");
    item.setActionCommand("_quicksort");
    item.addActionListener(this);
    menu.add(item);
    
    menu.add(new JSeparator());
    
    item = new JMenuItem("About...");
    item.setActionCommand("about");
    item.addActionListener(this);
    menu.add(item);
    
    item = new JMenuItem("Exit");
    item.setActionCommand("exit");
    item.addActionListener(this);
    menu.add(item);
    
    menu = new JMenu("Visualization");
    menuBar.add(menu);
    
    item = new JMenuItem("Pause");
    item.setActionCommand("!pause");
    item.addActionListener(this);
    menu.add(item);
    
    item = new JMenuItem("Resume");
    item.setActionCommand("!resume");
    item.addActionListener(this);
    menu.add(item);
    
    menu.add(new JSeparator());
    
    item = new JMenuItem("Slower");
    item.setActionCommand("!slower");
    item.addActionListener(this);
    menu.add(item);
    
    item = new JMenuItem("Faster");
    item.setActionCommand("!faster");
    item.addActionListener(this);
    menu.add(item);
    
    setJMenuBar(menuBar);
    
    // ... end menu bar setup
    
    
    setSize(INITIAL_SIZE, INITIAL_SIZE);
    
    // display initial panel
    DisplayPanel dp = new DisplayPanel();
    dp.createViz("_welcome");   
    setContentPane(dp);
    
    setVisible(true);
    toFront();
  }
  
  // this function gets called whenever a menu option is used
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    
    if (cmd.startsWith("_")) {
      DisplayPanel dp = (DisplayPanel) getContentPane();
      dp.terminate();
      
      dp = new DisplayPanel();
      setContentPane(dp);
      validate();
      dp.createViz(cmd);    //called after validate so that contentpane has a size
      repaint();
    }
    
    else if (cmd.startsWith("!")) {
      DisplayPanel dp = (DisplayPanel) getContentPane();
      dp.menuAction(cmd);
    }
    
    else if (cmd.equals("about")) {
      JOptionPane.showMessageDialog(this,
                                    "Algorithm Visualization program\nCOSC 102, Colgate University\nImplemented in Java using the Swing GUI Framework",
                                    "About SortViz",
                                    JOptionPane.PLAIN_MESSAGE);
    }
    
    else if (cmd.equals("exit")) {
      dispose();
      System.exit(0);
    }
    
    return;
    
  }
  
  public static void main (String[] args) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        new SortViz();
      }
    });
  }
  
}
