/* WalkViz.java
 * 
 An animator panel that just moves rectangles randomly around the window.
 (c) 2013 Vijay Ramachandran
 Modified by Elodie Fourquet
 COSC 102, Colgate University
 
 First part of the lab: experiment with this file to understand the code
 Do the three following modifications
 1. Read code: can the animation be over?
 change it to finish earlier
 2. Make the square bigger
 3. Change the drawing so that there are bars undernies
 as shown in the provided image: the squares still are shown but below
 are bars that extend short of the bottom of the window
 However, feel free to use it as a template for your mergesort and quicksort implementations.
 */

// import statements to get access to the graphics stuff
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;


// class declaration -- MUST extend AlgViz
public class WalkViz extends AlgViz {
  
  // --------- these are helper "inner" classes -----------
  
  // this one is used to maintain the state for each random walker in the animation
  private static class Walker {
    
    int x;  // its x position
    int y;  // its y position
    int size;   // its size
    Color c;    // its color
    
    /* make a new walker with given size
     * 
     we use the window height and size to help place the walker at a random location
     within those bounds
     */
    Walker(int size, int initialWidth, int initialHeight) {
      this.size = size;
      
      // set this walker's location to be random within the window
      x = (int) (Math.random() * (initialWidth/2) + (initialWidth/4));
      y = (int) (Math.random() * (initialHeight/2) + (initialHeight/4));
      
      // set the walker's color to be random
      c = new Color((int) (Math.random() * 128), (int) (Math.random() * 128), (int) (Math.random() * 128));
      
    }
    
    /* this is the function that gets called from vizPaint
     it draws this walker in the window */
    void draw(Graphics g, int winCurrentWidth, int winCurrentHeight) {
      g.setColor(c);
      g.fillRect(x, y, size, size);
    }
    
  }
  
  // this one is used to keep track of the graphics changes, for animation
  private static class WalkerEvent {
    int w;  // basically, which walker randomly moves?
    
    WalkerEvent(int w)
    { this.w = w; }
  }
  
  // ---------- end of private helper classes ---------- 
  
  
  // here are some constants that be changed to affect the behavior of this animation
  private final int number_of_walkers = 3;    // number of moving units
  private final int number_of_moves = 500;    // number of times the units can move
  private final int walker_size = 5;          // size of the unit in the window
  
  private LinkedList<WalkerEvent> eventQueue = new LinkedList<WalkerEvent>();
  // this is what tracks the animation events
  
  Walker[] walkerArray = new Walker[number_of_walkers];
  // instances of the walkers that move around
  
  // constructor -- by the end of this, the items that will be initially drawn should exist
  // the width and height of the window are not stored as the window is dynamic: it can be resized
  // instance methods will be passed the new current width and height values
  public WalkViz(int width, int height) {
    
    // initialize the walker instances
    for (int i = 0; i < number_of_walkers; i++)
      walkerArray[i] = new Walker(walker_size, width, height);
  }
  
  // this method is what actually performs the algorithm
  // in your case, this will be a sorting algorithm
  // regardless, it should generate animation events that go in the queue
  public void run() {
    for (int j = 0; j < number_of_moves; j++)
      for (int i = 0; i < number_of_walkers; i++)
      eventQueue.add(new WalkerEvent(i));
  }
  
  // this method is called by DisplayPanel
  // it's what draws the contents of the window
  public void vizPaint(Graphics g, int width, int height) {
    /*  we designed each walker to draw itself,
     so let's just call the draw function of each,
     passing along the graphics object needed
     to make changes to the window
     
     */
    
    for (int i = 0; i < number_of_walkers; i++)
      walkerArray[i].draw(g, width, height);
  }
  
  
  // this method is called by the timer in DisplayPanel
  // basically, it should respond to each animation event in the queue
  public void vizTimer(int width, int height) {
    
    /* don't do anything if the animation list is empty */
    if (eventQueue.size() <= 0)
      return;
    
    /* get the event at the beginning of the list,
     we.w will tell us which walker we are moving */
    WalkerEvent we = eventQueue.remove();
    Walker w = walkerArray[we.w];
    
    // adjusts the position of the walker number contained in the event
    int direction = (int) (Math.random() * 5 + 0);  // random number between 0 (incl.) and 5 (excl.)
    
    // based on the random number, move in a given direction        
    switch (direction) {
      case 1: // left
        w.x -= walker_size;
        if (w.x < 0)
          w.x = 0;
        break;
        
      case 2: // right
        w.x += walker_size;
        if (w.x > width - walker_size)
          w.x = width - walker_size;
        break;
        
      case 3: // up
        w.y -= walker_size;
        if (w.y < 0)
          w.y = 0;
        break;
        
      case 4: // down
        w.y += walker_size;
        if (w.y > height - walker_size)
          w.y = height - walker_size;
        break;
        
      default: // no movement
        break;
    }
    
  }
}

